# -*- coding: utf-8 -*-

import law
import luigi
import os
import fnmatch
from subprocess import PIPE
from law.util import interruptable_popen

from CopyTables import CopyTables

from analysis.framework import Task

class Combine(Task):
  my_debug = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')
  tags_dir = luigi.Parameter(default='tags')

  cores = luigi.Parameter()
  combine_ini = luigi.Parameter()

  def requires(self):
    return {
      'tables': CopyTables()
    }

  def output(self):
    target = '{}/{}/{}.{}.combine.log'.format(self.tags_dir, self.name, self.process, self.name)
    if self.my_debug:
      print('[Combine]: DEBUG: Local target is {}'.format(target))
    return law.LocalFileTarget(target)

  def run(self):
    prevdir = os.getcwd()
    combine_ini = os.path.abspath(self.combine_ini)
    self.output().parent.touch()

    try:
      os.chdir('{}/{}'.format(self.merge_dir, self.name))

      if self.my_debug:
        print('[Combine]: DEBUG: Starting global nnlojet-combine run ...')
      code, out, error = interruptable_popen(['nnlojet-combine.py', '-C', combine_ini, '-j', self.cores], stdout=PIPE, stderr=PIPE)
      if self.my_debug:
        print('[Combine]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
        print('[Combine]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
        print('[Combine]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
      if (code != 0):
        raise Exception('[Combine] ERROR! nnlojet-combine returned error {} and non-zero exit status {}'.format(error, code))

      os.chdir(prevdir)
      self.output().dump(out, formatter="text")

    except:
      os.chdir(prevdir)
      raise
