# -*- coding: utf-8 -*-

import law
import luigi
import os
import re

from FastWarm import FastWarm

from analysis.framework import Task

class CopyFastWarm(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)
  force = luigi.BoolParameter(default=False)

  warm_dir  = luigi.Parameter(default='warmups')

  channels = luigi.Parameter()

  def create_branch_map(self):
    return FastWarm.req(self).branch_map

  def workflow_requires(self):
    if self.force:
      return {"fastwarm": FastWarm.req(self, acceptance=0)}
    else:
      return {"fastwarm": FastWarm.req(self)}

  def output(self):
    basename = FastWarm(branch = self.branch).output().basename
    outfile = re.sub('.gz$', '', basename)
    outfile = re.sub('.tar$', '', outfile) + '.log'
    return law.LocalFileTarget('{}/{}/{}/{}'.format(self.warm_dir, 'fastwarm', self.name, outfile))

  def run(self):
    self.output().parent.touch()
    prevdir = os.getcwd()
    os.chdir(self.output().parent.path)
    input = FastWarm(branch = self.branch).output()
    if not input.exists():
      print('[CopyFastWarm]: Warning! FastWarm result not found, skipping branch: {}'.format(self.branch))
    else:
      input.load('')
    os.chdir(prevdir)
