# -*- coding: utf-8 -*-

import law
import luigi
import os
import re

from MergeFastWarm import MergeFastWarm

from analysis.framework import Task

class CopyMergeFastWarm(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  warm_dir  = luigi.Parameter(default='warmups')

  observables = luigi.ListParameter()

  def create_branch_map(self):
    branchmap = {}
    i = 0
    for observable in self.observables:
      branchmap[i] = {
        'observable': observable
      }
      i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'mergefastwarm': MergeFastWarm()
    }

  def output(self):
    return law.LocalFileTarget('{}/{}/{}/{}.{}.{}'.format(self.warm_dir, 'mergefastwarm', self.name, self.process, self.branch_data['observable'], 'wrm'))

  def run(self):
    self.output().parent.touch()
    prevdir = os.getcwd()
    os.chdir(self.output().parent.path)
    input = MergeFastWarm().output()
    if not input.exists():
      raise Exception('[CopyMergeFastWarm]: ERROR! Input file not found: {}'.format(input))
    else:
      input.load('')
    os.chdir(prevdir)
