# -*- coding: utf-8 -*-

import law
import luigi
import os
import re

from Rewarmup import Rewarmup

from analysis.framework import Task

class CopyRewarmup(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  warm_dir  = luigi.Parameter(default='warmups')

  jobnameext = luigi.Parameter()
  cutoff = luigi.Parameter()
  channels = luigi.Parameter()
  kinetags = luigi.DictParameter()

  def create_branch_map(self):
    i = 0
    branchmap = {}
    for j, channel in enumerate(self.channels.split()):
      branchmap[i] = {
        'channel': channel,
        'kinetag': self.kinetags[channel]
      }
      i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'rewarmup': Rewarmup()
    }

  def output(self):
    return law.LocalFileTarget('{}/{}/{}/{}.{}-{}.{}.{}'.format(self.warm_dir, 'rewarmup', self.name, self.process, self.branch_data['channel'], self.jobnameext, self.cutoff, self.branch_data['kinetag']))

  def run(self):
    self.output().parent.touch()
    prevdir = os.getcwd()
    os.chdir(self.output().parent.path)
    input = Rewarmup().output()['collection'].targets[self.branch]
    if not input.exists():
      raise Exception('[CopyRewarmup]: ERROR! Input file not found: {}'.format(input))
    else:
      if self.my_debug:
        print('[CopyRewarmup]: DEBUG: Unpacking target {} to local directory {}/{}/{}'.format(input, self.warm_dir, 'rewarmup', self.name))
      input.load('')
      os.chdir(prevdir)
