# -*- coding: utf-8 -*-

import law
import luigi
import os
import re

from FastProd import FastProd

from analysis.framework import Task

class CopyTables(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)
  force = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')

  channels = luigi.Parameter()

  def create_branch_map(self):
    return FastProd.req(self).branch_map

  def workflow_requires(self):
    if self.force:
      return {"fastprod": FastProd.req(self, acceptance=0)}
    else:
      return {"fastprod": FastProd.req(self)}

  def output(self):
    basename = FastProd(branch = self.branch).output().basename
    outfile = re.sub('.gz$', '', basename)
    outfile = re.sub('.tar$', '', outfile) + '.log'
    target  = '{}/{}/{}/{}'.format(self.merge_dir, self.name, self.branch_data['channel'], outfile)
    if self.my_debug:
      print('[CopyTables]: DEBUG: Local target is {}'.format(target))
    return law.LocalFileTarget(target)

  def run(self):
    self.output().parent.touch()
    prevdir = os.getcwd()
    os.chdir(self.output().parent.path)
    input = FastProd(branch = self.branch).output()
    if not input.exists():
      print('[CopyTables]: Warning! FastProd result not found, skipping branch {}'.format(self.branch))
    else:
      input.load('')
    os.chdir(prevdir)
