# -*- coding: utf-8 -*-

import law
import luigi
import os
import re

from Warmup import Warmup

from analysis.framework import Task

class CopyWarmup(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)
  force = luigi.BoolParameter(default=False)

  warm_dir  = luigi.Parameter(default='warmups')

  jobnameext = luigi.Parameter()
  cutoff = luigi.Parameter()
  channels = luigi.Parameter()
  kinetags = luigi.DictParameter()

  def create_branch_map(self):
    branchmap = {}
    for i, channel in enumerate(self.channels.split()):
      branchmap[i] = {
        'index': i,
        'channel': channel,
        'kinetag': self.kinetags[channel]
      }
    return branchmap

  def workflow_requires(self):
    if self.force:
      return {"warmup": Warmup.req(self, acceptance=0)}
    else:
      return {"warmup": Warmup.req(self)}

  def requires(self):
    return {
      'warmup': Warmup(branch = self.branch_data['index'])
    }

  def output(self):
    return law.LocalFileTarget('{}/{}/{}/{}.{}-{}.{}.{}'.format(self.warm_dir, 'warmup', self.name, self.process, self.branch_data['channel'], self.jobnameext, self.cutoff, self.branch_data['kinetag']))

  def run(self):
    self.output().parent.touch()
    prevdir = os.getcwd()
    os.chdir(self.output().parent.path)
    input = Warmup().output()['collection'].targets[self.branch]
    if not input.exists():
      raise Exception('[CopyWarmup]: ERROR! Input file not found: {}'.format(input))
    else:
      if self.my_debug:
        print('[CopyWarmup]: DEBUG: Unpacking target {} to local directory {}/{}/{}'.format(input, self.warm_dir, 'warmup', self.name))
      input.load('')
      os.chdir(prevdir)
