# -*- coding: utf-8 -*-

import law
import luigi
import glob
import os
import re

from law.util import interruptable_popen
from subprocess import PIPE

from CopyTables import CopyTables

from analysis.framework import Task

class FnloCppread(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)
  force = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')
  tags_dir = luigi.Parameter(default='tags')

  pdf = luigi.Parameter()
  scalecombs = luigi.Parameter()
  ascode = luigi.Parameter()
  norm = luigi.Parameter()
  scale = luigi.Parameter()

  jobnameext = luigi.Parameter()
  observables = luigi.ListParameter()
  kinetags = luigi.DictParameter()

  def create_branch_map(self):
    return CopyTables.req(self).branch_map

  def workflow_requires(self):
    if self.force:
      return {"copytables": CopyTables.req(self, acceptance=0)}
    else:
      return {"copytables": CopyTables.req(self)}

  def output(self):
    target = '{}/{}/{}/{}.{}.{}.cppread.s{}.tag'.format(self.tags_dir, self.name, self.branch_data['channel'], self.process, self.branch_data['channel'], self.name, self.branch_data['seed'])
    if self.my_debug:
      print('[FnloCppread]: DEBUG: Local target is {}'.format(target))
    return law.LocalFileTarget(target)

  def run(self):
    try:
      tableglob = self.merge_dir + '/' + self.name + '/' + self.branch_data['channel'] + '/*.s' + self.branch_data['seed'] + '.tab.gz'
      iscl    = 0
      scls    = self.scale.split()
      sclcoms = self.scalecombs.split()
      if self.my_debug:
        print('[FnloCppread]: DEBUG: Table glob is {}'.format(tableglob))
      tables = glob.glob(tableglob)
      if not tables:
        raise Exception('[FnloCppread]: ERROR! Could not find tables to evaluate for branch {}'.format(self.branch))
      else:
        ierr = 0
        for table in tables:
          if len(scls)*len(sclcoms) > 1:
            for i, observable in enumerate(self.observables):
              if observable in table:
                iscl = i
                if self.my_debug:
                  print('[FnloCppread]: DEBUG: Table matches observable {} with index {}'.format(observable,iscl))
          logfile = re.sub('.gz$', '', table)
          logfile = re.sub('.tab$', '', logfile) + '.log'
          if self.my_debug:
            print('[FnloCppread]: DEBUG: Dealing with table {}'.format(table))
            print('[FnloCppread]: DEBUG: The log filename is {}'.format(logfile))

          scl = scls[0]
          if len(scls) > 1:
            scl = scls[iscl]
          sclcom = sclcoms[0]
          if len(sclcoms) > 1:
            sclcom = sclcoms[iscl]
          if self.my_debug:
            print('[FnloCppread]: DEBUG: Investigating central scale {} for scale combinations {}'.format(scl,sclcom))

          code, out, error = interruptable_popen(['fnlo-tk-cppread', table, self.pdf, sclcom, self.ascode, self.norm, scl], stdout=PIPE, stderr=PIPE)
          if self.my_debug:
            print('[FnloCppread]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
            print('[FnloCppread]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
            print('[FnloCppread]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
          if (code != 0):
            ierr += 1
            print('[FnloCppread]: ERROR! Table {} gave message {} and exit code {}. SKIPPED!'.format(table, error, code))
          else:
            with open(logfile, 'w') as outfile:
              outfile.write(out)
          if ierr > 0:
            raise Exception('[FnloCppread]: ERROR! Tables could not be evaluated for all observables for seed {} of branch {}. Please remove the whole job!'.format(self.branch_data['seed'], self.branch))
        self.output().touch()
    except:
      raise
