# -*- coding: utf-8 -*-

import law
import luigi
import os
import glob
import re

from subprocess import PIPE
from law.util import interruptable_popen

from MergeFinal import MergeFinal

from analysis.framework import Task

class FnloCppreadFinal(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')

  channels = luigi.Parameter()
  final_tables = luigi.DictParameter()
  observables = luigi.ListParameter()

  pdf = luigi.Parameter()
  scalecombs = luigi.Parameter()
  ascode = luigi.Parameter()
  norm = luigi.Parameter()
  scale = luigi.Parameter()

  def create_branch_map(self):
    branchmap = {}
    i = 0
    for observable in self.observables:
      for channel in self.channels.split():
        branchmap[i] = {
          'observable': observable,
          'channel': channel
        }
        i += 1
      for channel in self.final_tables:
        branchmap[i] = {
          'observable': observable,
          'channel': channel
        }
        i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'mergefinal': MergeFinal()
    }

  def requires(self):
    return MergeFinal()

  def output(self):
    return law.LocalFileTarget('{}/{}/Combined/Final/{}.{}.{}.log'.format(self.merge_dir, self.name, self.process, self.branch_data['channel'], self.branch_data['observable']))

  def run(self):
    try:
      self.output().parent.touch()
      logfile = self.output().path
      table = re.sub('.log', '', logfile ) + '.tab.gz'
      if self.my_debug:
        print('[FnloCppreadFinal]: DEBUG: Table file is {}'.format(table))

      iscl    = 0
      scls    = self.scale.split()
      sclcoms = self.scalecombs.split()
      if len(scls)*len(sclcoms) > 1:
        for i, observable in enumerate(self.observables):
          if observable in table:
            iscl = i
            if self.my_debug:
              print('[FnloCppread]: DEBUG: Table matches observable {} with index {}'.format(observable,iscl))

      scl = scls[0]
      if len(scls) > 1:
        scl = scls[iscl]
      sclcom = sclcoms[0]
      if len(sclcoms) > 1:
        sclcom = sclcoms[iscl]
      if self.my_debug:
        print('[FnloCppreadFinal]: DEBUG: Investigating central scale {} for scale combinations {}'.format(scl,sclcom))

      code, out, error = interruptable_popen(['fnlo-tk-cppread', table, self.pdf, sclcom, self.ascode, self.norm, scl],stdout=PIPE, stderr=PIPE)
      if self.my_debug:
        print('[FnloCppreadFinal]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
        print('[FnloCppreadFinal]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
        print('[FnloCppreadFinal]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
      if (code != 0):
        raise Exception('[FnloCppreadFinal] ERROR! fnlo-tk-cppread returned message {} and non-zero exit code {}'.format(error, code))
      with open(logfile, 'w') as outfile:
        outfile.write(out)

    except:
      raise
