# -*- coding: utf-8 -*-

import law
import luigi
import os
from law import util
from subprocess import PIPE

from FnloCppreadFinal import FnloCppreadFinal
from Recombine import Recombine

from analysis.framework import Task

class KfacComparison(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')
  plots_dir = luigi.Parameter(default='plots')

  orders = luigi.ListParameter()
  observables = luigi.ListParameter()

  def create_branch_map(self):
    branchmap = {}
    i = 0
    for observable in self.observables:
      branchmap[i] = {
        'observable': observable
      }
      i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'fnlocppread': FnloCppreadFinal(),
      'recombine': Recombine()
    }

  def output(self):
    return law.LocalDirectoryTarget('{}/{}/KfacComparison/{}'.format(self.plots_dir, self.name, self.branch_data['observable']))

  def run(self):
    try:
      with self.output().temporary_path() as self.temp_output_path:
        os.mkdir(self.temp_output_path)

        for order in self.orders:
          datfile = '{}/{}/Combined/Final/{}.{}.{}.dat'.format(self.merge_dir, self.name, self.process, order, self.branch_data['observable'])
          outfile = '{}/{}.{}.{}'.format(self.temp_output_path, self.process, order, self.branch_data['observable'])

          code, out, error = util.interruptable_popen(['fastnnlo_kfaccomp.py', datfile, '-f', outfile], stdout=PIPE, stderr=PIPE)
          if self.my_debug:
            print('[KfacComparison]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
            print('[KfacComparison]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
            print('[KfacComparison]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
          if (code != 0):
            raise Exception('[KfacComparison]: {} exitcode: {}'.format(error, code))
    except:
      os.rmdir(self.temp_output_path)
      raise
