# -*- coding: utf-8 -*-

import luigi
import law
import os
import re
import shutil

from subprocess import PIPE
from law.util import interruptable_popen

from Combine import Combine

from analysis.framework import Task, HTCondorWorkflow

class MergeFastProd(Task, HTCondorWorkflow, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')

  channels = luigi.Parameter()
  weightorder = luigi.Parameter()
  observables = luigi.ListParameter()

  def create_branch_map(self):
    channels = self.channels.split()
    i = 0
    branchmap = {}
    for channel in channels:
      for obs in self.observables:
        branchmap[i] = {
          'obs': obs,
          'channel': channel
        }
        i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'combine': Combine()
    }

  def requires(self):
    return Combine()

  def output(self):
    return law.LocalFileTarget('{}/{}/Combined/Final/{}.{}.{}.tab.gz'.format(self.merge_dir, self.name, self.process, self.branch_data['channel'], self.branch_data['obs']))

  def run(self):
    self.output().parent.touch()

    outfile = self.output().path
    logfile = re.sub('.tab.gz$', '.merge2.log', outfile)

    weightfile = '{}/{}/Combined/Final/{}.{}.APPLfast.txt'.format(self.merge_dir, self.name, self.weightorder, self.branch_data['obs'])
    tablenames = '{}/{}/{}/*.{}.s*.tab.gz'.format(self.merge_dir, self.name, self.branch_data['channel'], self.branch_data['obs'])
    cmdline    = 'fnlo-tk-merge2 -w NNLOJET {} {} {}'.format(weightfile, tablenames, outfile)
    if self.my_debug:
      print('[MergeFastProd]: DEBUG: The tablenames argument is: {}'.format(tablenames))
      print('[MergeFastProd]: DEBUG: The commandline reads: {}'.format(cmdline))

    code, out, error = interruptable_popen('fnlo-tk-merge2 -w NNLOJET {} {} {}'.format(weightfile, tablenames, outfile), shell=True, stdout=PIPE, stderr=PIPE)
    if self.my_debug:
      print('[MergeFastProd]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
      print('[MergeFastProd]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
      print('[MergeFastProd]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
    with open(logfile, 'w') as outfile:
      outfile.write(out)
    if (code != 0):
      raise Exception('[MergeFastProd]: ERROR! fnlo-tk-merge2 returned non-zero exit status {}'.format(code))
