# -*- coding: utf-8 -*-

import luigi
import six
import glob
import os
import shutil
from fnmatch import fnmatch
from subprocess import PIPE

from law.util import interruptable_popen

from FastWarm import FastWarm

from analysis.framework import Task

class MergeFastWarm(Task):
  my_debug = luigi.BoolParameter(default=False)

  process = luigi.Parameter()
  observables = luigi.ListParameter()

  def requires(self):
    return FastWarm()

  def output(self):
    return self.remote_target('{}.{}.fastwarm.tar.gz'.format(self.process, self.name))

  def run(self):
    dirpath = 'tmpdir'
    prevdir = os.getcwd()

    try:
      os.mkdir(dirpath)
      os.chdir(dirpath)
      self.output().parent.touch()
      inputs = self.input()['collection'].targets
      for inp in six.itervalues(inputs):
        try:
          inp.load('')
        except:
          print('[MergeFastWarm]: WARNING: File {} does not exist!'.format(inp))
      # Remove fastwarm logs and runcards before output preps
      os.system('rm *fastwarm*.log')
      os.system('rm *fastwarm*.run')
      for obs in self.observables:
        wrmfile = '{}.{}.wrm'.format(self.process, obs)
        logfile = '{}.{}.addwarmup.log'.format(self.process, obs)
        if self.my_debug:
          print('[MergeFastWarm]: DEBUG: wrmfile is {}'.format(wrmfile))
          print('[MergeFastWarm]: DEBUG: logfile is {}'.format(logfile))
        code, out, error = interruptable_popen(['fnlo-add-warmup_v23.pl', '-d', '-v', '2.4', '-w', '.', '-o', wrmfile, obs], stdout=PIPE, stderr=PIPE)
        if self.my_debug:
          print('[MergeFastWarm]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
          print('[MergeFastWarm]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
          print('[MergeFastWarm]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
        with open(logfile, 'w') as outfile:
          outfile.write(out)
        if (code != 0):
          raise Exception('[MergeFastWarm]: ERROR! fnlo-add-warmup returned non-zero exit status {}'.format(code))
        # Remove unmerged wrm files before output preps
        os.system('rm {}.*.{}.s*.wrm'.format(self.process, obs))

      tarfilter = lambda n : n if (fnmatch(n.name, '*.wrm') or fnmatch(n.name, '*.log')) else None
      self.output().dump(os.getcwd(), filter=tarfilter)

    finally:
      # Clean up
      os.chdir(prevdir)
      shutil.rmtree('tmpdir')
