# -*- coding: utf-8 -*-

import law
import luigi
import os
import glob
from law.util import interruptable_popen
from subprocess import PIPE

from CopyTables import CopyTables

from analysis.framework import Task

class PlotRuntime(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)
  force = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')
  plots_dir = luigi.Parameter(default='plots')

  channels = luigi.Parameter()
  jobnameext = luigi.Parameter()

  def create_branch_map(self):
    branchmap = {}
    for i, channel in enumerate(self.channels.split()):
      branchmap[i] = {
        'index': i,
        'channel': channel
      }
    i += 1
    branchmap[i] = {'index': i, 'channel': 'ALL'}
    return branchmap

  def workflow_requires(self):
    if self.force:
      return {"copytables": CopyTables.req(self, acceptance=0)}
    else:
      return {"copytables": CopyTables.req(self)}

  def output(self):
    target = '{}/{}/PlotRuntime/{}.{}-{}.runtime.png'.format(self.plots_dir, self.name, self.process, self.branch_data['channel'], self.jobnameext)
    if self.my_debug:
      print('[PlotRuntime]: DEBUG: Local target is {}'.format(target))
    return law.LocalFileTarget(target)

  def run(self):
    self.output().parent.touch()
    if self.my_debug:
      print('[PlotRuntime]: DEBUG: Created target directory.')
    outpath = self.output().parent.path
    if self.my_debug:
      print('[PlotRuntime]: DEBUG: Out path: {}'.format(outpath))

    mychann = self.branch_data['channel']
    plotnam = self.process + '.' + mychann + '-' + self.jobnameext
    myglob  = mychann
    if mychann == 'ALL':
      myglob = '*'
    logglob = self.merge_dir + '/' + self.name + '/' + myglob + '/' + self.process + '.' + myglob + '.*.fastprod.s*.log'
    if self.my_debug:
      print('[PlotRuntime]: DEBUG: Log glob: {}'.format(logglob))

    mycmd = ['fastnnlo_runtime.py', str(logglob), '-o', outpath, '-f', plotnam, '--CPUtime']
    if mychann == 'ALL':
      mycmd.append('--Events')

    try:
      code, out, error = interruptable_popen(mycmd, stdout=PIPE, stderr=PIPE)
      if self.my_debug:
        print('[PlotRuntime]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
        print('[PlotRuntime]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
        print('[PlotRuntime]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
      if (code != 0 or error):
        raise Exception('[PlotRuntime]: ERROR! fastnnlo_runtime.py returned error {} and non-zero exit status {}'.format(error, code))

    except:
      # Clean up failure
      if self.output().exists():
        self.output().remove()
      raise Exception('[PlotRuntime]: ERROR! Something went wrong. Cleaned output (if existing).')
