# -*- coding: utf-8 -*-

import law
import luigi
import os
import glob
import re

from subprocess import PIPE
from law.util import interruptable_popen

from Combine import Combine

from analysis.framework import Task

class Recombine(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')

  channels = luigi.Parameter()
  observables = luigi.ListParameter()

  cores = luigi.Parameter()
  combine_ini = luigi.Parameter()

  def create_branch_map(self):
    branchmap = {}
    i = 0
    for observable in self.observables:
      for channel in self.channels.split():
        branchmap[i] = {
          'observable': observable,
          'channel': channel
        }
        i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'combine': Combine()
    }

  def requires(self):
    return Combine()

  def output(self):
    return law.LocalFileTarget('{}/{}/Combined/Final/{}.{}.{}.dat'.format(self.merge_dir, self.name, self.process, self.branch_data['channel'], self.branch_data['observable']))

  def run(self):
    prevdir = os.getcwd()
    combine_ini = os.path.abspath(self.combine_ini)
    self.output().parent.touch()

    try:
      os.chdir('{}/{}'.format(self.merge_dir, self.name))

      weightfile = 'Combined/Final/{}.{}.APPLfast.txt'.format(self.branch_data['channel'], self.branch_data['observable'])
      targetfile = 'Combined/Final/{}.{}.{}.dat'.format(self.process, self.branch_data['channel'], self.branch_data['observable'])
      if self.my_debug:
        print('[Recombine]: DEBUG: Starting recombining using weight file {}'.format(weightfile))
      code, out, error = interruptable_popen(['nnlojet-combine.py -C {} --APPLfast {} > {}'.format(combine_ini, weightfile, targetfile)], shell=True, stdout=PIPE, stderr=PIPE)
      if self.my_debug:
        print('[Recombine]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
        print('[Recombine]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
        print('[Recombine]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
      if (code != 0):
        raise Exception('[Recombine] ERROR! nnlojet-combine returned error {} and non-zero exit status {}'.format(error, code))

      os.chdir(prevdir)
#      self.output().dump(job_out, formatter="text")

    except:
      os.chdir(prevdir)
      raise
