# -*- coding: utf-8 -*-

import law
import luigi
import os

from law.util import interruptable_popen
from subprocess import PIPE

from FnloCppreadFinal import FnloCppreadFinal

from analysis.framework import Task

class ScaleUncertainty(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')
  plots_dir = luigi.Parameter(default='plots')

  orders = luigi.ListParameter()
  observables = luigi.ListParameter()
  pdf = luigi.Parameter()
  scaleindex = luigi.Parameter()
  datfiles = luigi.ListParameter()

  def create_branch_map(self):
    branchmap = {}
    i = 0
    for order in self.orders:
      branchmap[i] = {
        'order': order
      }
      i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'fnlocppread': FnloCppreadFinal(self)
    }

  def output(self):
    return law.LocalDirectoryTarget('{}/{}/ScaleUncertainty/{}'.format(self.plots_dir, self.name, self.branch_data['order']))

  def run(self):
    try:
      with self.output().temporary_path() as self.temp_output_path:
        os.mkdir(self.temp_output_path)

        for obs in self.observables:
          for scli in self.scaleindex.split():
            for stunc in self.datfiles:

              if stunc == '':
                ext = '.tab.gz'
              elif stunc == '0':
                ext = '_v25.tab.gz'
              else:
                print('[ScaleUncertainty]: ERROR: Unknown datfiles option:\n{}\n'.format(stunc))
                exit(22)

              tabfile = '{}/{}/Combined/Final/{}.{}.{}{}'.format(self.merge_dir, self.name, self.process, self.branch_data['order'], obs, ext)
              outfile = '{}/{}.{}.{}.statunc{}'.format(self.temp_output_path, self.process, self.branch_data['order'], obs, stunc)

              if self.my_debug:
                cmd = 'fastnnlo_scaleunc.py -a -d {} -p {} -s {} -f {} {}'.format(stunc,self.pdf,str(scli),outfile,tabfile)
                print('[ScaleUncertainty]: DEBUG: Printing subprocess command:\n{}'.format(cmd))
              code, out, error = interruptable_popen(['fastnnlo_scaleunc.py', '-a', '-d', stunc, '-p', self.pdf, '-s', str(scli), '-f', outfile, tabfile], stdout=PIPE, stderr=PIPE)

              if self.my_debug:
                print('[ScaleUncertainty]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
                print('[ScaleUncertainty]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
                print('[ScaleUncertainty]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
              if (code != 0):
                raise Exception('[ScaleUncertainty]: {} exitcode: {}'.format(error, code))

            # Only plot NLO & NNLO
            #            code, out, error = interruptable_popen(['fastnnlo_scaleunc.py', '-a', '-d', '-o', 'NLO,NNLO', '-p', self.pdf, '-s', str(scli), '-f', outfile, tabfile], stdout=PIPE, stderr=PIPE)
            #            if self.my_debug:
            #              print('[ScaleUncertainty]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
            #              print('[ScaleUncertainty]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
            #              print('[ScaleUncertainty]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
            #            if (code != 0):
            #              raise Exception('[ScaleUncertainty]: {} exitcode: {}'.format(error, code))
    except:
      os.rmdir(self.temp_output_path)
      raise
