# -*- coding: utf-8 -*-

import law
import luigi
import os

from law.util import interruptable_popen
from subprocess import PIPE

from Recombine import Recombine

from analysis.framework import Task

class StatUncertainty(Task, law.LocalWorkflow):
  my_debug = luigi.BoolParameter(default=False)

  merge_dir = luigi.Parameter(default='grids')
  plots_dir = luigi.Parameter(default='plots')

  orders = luigi.ListParameter()
  observables = luigi.ListParameter()

  def create_branch_map(self):
    branchmap = {}
    i = 0
    for order in self.orders:
      branchmap[i] = {
        'order': order
      }
      i += 1
    return branchmap

  def workflow_requires(self):
    return {
      'recombine': Recombine()
    }

  def output(self):
    return law.LocalDirectoryTarget('{}/{}/StatUncertainty/{}'.format(self.plots_dir, self.name, self.branch_data['order']))

  def run(self):
    try:
      with self.output().temporary_path() as self.temp_output_path:
        os.mkdir(self.temp_output_path)

        for obs in self.observables:
          tabfile = '{}/{}/Combined/Final/{}.{}.{}.tab.gz'.format(self.merge_dir, self.name, self.process, self.branch_data['order'], obs)
          outfile = '{}/{}.{}.{}'.format(self.temp_output_path, self.process, self.branch_data['order'], obs)
          if self.my_debug:
            print('[StatUncertainty]: DEBUG: Investigating table: {}'.format(tabfile))
            print('[StatUncertainty]: DEBUG: Outfile name is: {}'.format(outfile))

          code, out, error = interruptable_popen(['fastnnlo_statunc.py', '-o', self.branch_data['order'], '--title', obs, '-f', outfile, tabfile], stdout=PIPE, stderr=PIPE)
          if self.my_debug:
            print('[StatUncertainty]: DEBUG: Printing subprocess code:\n{}\n'.format(code))
            print('[StatUncertainty]: DEBUG: Printing subprocess output:\n{}\n'.format(out))
            print('[StatUncertainty]: DEBUG: Printing subprocess error:\n{}\n'.format(error))
          if (code != 0):
            raise Exception('[StatUncertainty]: {} exitcode: {}'.format(error, code))
    except:
      os.rmdir(self.temp_output_path)
      raise
